/*=========================================================================

Program:   BlueBerry Platform
Language:  C++
Date:      $Date: 2010-06-16 11:50:11 +0200 (Wed, 16 Jun 2010) $
Version:   $Revision: 23790 $

Copyright (c) German Cancer Research Center, Division of Medical and
Biological Informatics. All rights reserved.
See MITKCopyright.txt or http://www.mitk.org/copyright.html for details.

This software is distributed WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the above copyright notices for more information.

=========================================================================*/

#ifndef __BERRY_TEST_EXPRESSION_H__
#define __BERRY_TEST_EXPRESSION_H__

#include "../berryExpression.h"
#include "berryTypeExtensionManager.h"

#include "service/berryIConfigurationElement.h"
#include "berryObject.h"

#include "Poco/DOM/Element.h"

namespace berry {

class TestExpression : public Expression {

private:
  std::string fNamespace;
  std::string fProperty;
  std::vector<Object::Pointer> fArgs;
  Object::Pointer fExpectedValue;
  bool fForcePluginActivation;

  static const char PROP_SEP;
  static const std::string ATT_PROPERTY;
  static const std::string ATT_ARGS;
  static const std::string ATT_FORCE_PLUGIN_ACTIVATION;
  /**
   * The seed for the hash code for all test expressions.
   */
  static const std::size_t HASH_INITIAL;

  static TypeExtensionManager fgTypeExtensionManager;


public:

  TestExpression(SmartPointer<IConfigurationElement> element);

  TestExpression(Poco::XML::Element* element);

  TestExpression(const std::string& namespaze, const std::string& property,
      std::vector<Object::Pointer>& args, Object::Pointer expectedValue);

  TestExpression(const std::string& namespaze, const std::string& property,
      std::vector<Object::Pointer>& args, Object::Pointer expectedValue, bool forcePluginActivation);

  EvaluationResult Evaluate(IEvaluationContext* context);

  void CollectExpressionInfo(ExpressionInfo* info);

  bool operator==(Expression& object);

protected:

  std::size_t ComputeHashCode();

  //---- Debugging ---------------------------------------------------

  /* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
public:

  std::string ToString();

  //---- testing ---------------------------------------------------

  bool TestGetForcePluginActivation();

  static TypeExtensionManager& TestGetTypeExtensionManager();
};

} // namespace berry

#endif // __BERRY_TEST_EXPRESSION_H__
