/*=========================================================================
 
Program:   Medical Imaging & Interaction Toolkit
Language:  C++
Date:      $Date: 2010-01-28 18:32:03 +0100 (Thu, 28 Jan 2010) $
Version:   $Revision: 21147 $
 
Copyright (c) German Cancer Research Center, Division of Medical and
Biological Informatics. All rights reserved.
See MITKCopyright.txt or http://www.mitk.org/copyright.html for details.
 
This software is distributed WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the above copyright notices for more information.
 
=========================================================================*/

#ifndef MITK_CALCULATE_SEGMENTATION_VOLUME_H_INCLUDET_WAD
#define MITK_CALCULATE_SEGMENTATION_VOLUME_H_INCLUDET_WAD

#include "mitkSegmentationSink.h"
#include "MitkExtExports.h"
#include "mitkImageCast.h"

namespace mitk 
{

class MitkExt_EXPORT CalculateSegmentationVolume : public SegmentationSink
{
  public:
    
    mitkClassMacro( CalculateSegmentationVolume, SegmentationSink )
    mitkAlgorithmNewMacro( CalculateSegmentationVolume );
    
  protected:
    
    CalculateSegmentationVolume();  // use smart pointers
    virtual ~CalculateSegmentationVolume();

    virtual bool ReadyToRun();

    virtual bool ThreadedUpdateFunction(); // will be called from a thread after calling StartAlgorithm
    
    template < typename TPixel, unsigned int VImageDimension >
    void ItkImageProcessing( itk::Image< TPixel, VImageDimension >* itkImage, TPixel* dummy = NULL );

  private:

    unsigned int m_Volume;

    Vector3D m_CenterOfMass;
    Vector3D m_MinIndexOfBoundingBox;
    Vector3D m_MaxIndexOfBoundingBox;

};

} // namespace

#endif


