/*=========================================================================

Program:   Medical Imaging & Interaction Toolkit
Language:  C++
Date:      $Date: 2010-07-30 14:42:32 +0200 (Fri, 30 Jul 2010) $
Version:   $Revision: 24988 $

Copyright (c) German Cancer Research Center, Division of Medical and
Biological Informatics. All rights reserved.
See MITKCopyright.txt or http://www.mitk.org/copyright.html for details.

This software is distributed WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the above copyright notices for more information.

=========================================================================*/


#include "mitkBoundingObject.h"
#include "mitkBaseProcess.h"

mitk::BoundingObject::BoundingObject()
  : Surface(), m_Positive(true)
{
//  Initialize(1);

  /* bounding box around the unscaled bounding object */ 
  ScalarType bounds[6]={-1,1,-1,1,-1,1};  //{xmin,x_max, ymin,y_max,zmin,z_max}
  GetGeometry()->SetBounds(bounds);
  GetTimeSlicedGeometry()->UpdateInformation();
}

mitk::BoundingObject::~BoundingObject() 
{
} 

mitk::ScalarType mitk::BoundingObject::GetVolume()
{
  return 0.0;
}

void mitk::BoundingObject::FitGeometry(mitk::Geometry3D* aGeometry3D)
{
  GetGeometry()->SetIdentity();
  GetGeometry()->Compose(aGeometry3D->GetIndexToWorldTransform());

  if(aGeometry3D->GetImageGeometry())
  {
    mitk::Vector3D halfSpacingStep = aGeometry3D->GetSpacing()*0.5;
    GetGeometry()->SetOrigin(aGeometry3D->GetCenter()-halfSpacingStep);
  }
  else
  {
    GetGeometry()->SetOrigin(aGeometry3D->GetCenter());
  }

  mitk::Vector3D size;
  for(unsigned int i=0; i < 3; ++i)
    size[i] = aGeometry3D->GetExtentInMM(i)/2.0;
  GetGeometry()->SetSpacing( size );
  GetTimeSlicedGeometry()->UpdateInformation();
}
